# Filename:      consolenotify.py
# Purpose:       notify extension for use as commit hook within mercurial
# Authors:       grml-team (grml.org), (c) Michael Gebetsroither <gebi@grml.org>, (c) Michael Prokop <mika@grml.org>
# Bug-Reports:   see http://grml.org/bugs/
# License:       This file is licensed under the GPL v2.
# Latest change: Don Jun 14 23:01:40 CEST 2007 [mika]
################################################################################
# configuration example:
#
# [extension]
#   grmlhgext/consolenotify=
# [hooks]
#   commit.1url   = python:grmlhgext/consolenotify.urlhook
#   outgoing.1url = python:grmlhgext/consolenotify.urlhook
################################################################################

import os
from mercurial.node import *

def startswith(s, match):
    for i in match:
        if s.startswith(i):
            return True
    return False

debugtemplate = """\
defaulturl  = "%s"
defaultweb  = "%s"
defaultpush = "%s"
default     = "%s"
url         = "%s"\n"""

def urlhook(ui, repo, hooktype, node=None, source=None, **kwargs):
    try:
        mq_appdirty = repo.mq.applied_dirty
        mq_numpatches = len(repo.mq.applied)
    except:
        # hg clone <lokal> <new repos over ssh>
        # 'localrepository' object has no attribute 'mq'
        return
    ui.debug("repo.mq.applied_dirty=%s\n" % mq_appdirty)
    ui.debug("len(repo.mq.applied)=%s\n" % mq_numpatches)
    if not mq_appdirty and mq_numpatches == 0:
        defaulturl = ui.config("consolenotify", "default-url") or "http://hg.grml.org"
        defaultweb = ui.config("paths", "default-web") or defaulturl
        defaultp   = ui.config("paths", "default-push") or ""
        default    = ui.config("paths", "default") or ""

        # try to set dirname based on "default" or fall back to dirname
        # should avoid wrong URLs when using cloned repositories where
        # dirname is not same as repository name
        path = ui.config("paths", "default")
        dirname = os.path.basename(path) or os.path.basename(repo.root)

        url = defaulturl + '/' + dirname
        web = ('http://', 'https://')
        if startswith(defaultp, web):
            url = defaultp.rstrip('/')
        elif startswith(default, web) and defaultp == "":
            url = default.rstrip('/')
        elif startswith(default, web) or startswith(default, '/'):
            # quirks mode: defaultp==ssh and default=http/local
            # no hgweb from push known, using default-web instead
            # or defaulturl if not given.
            url = defaultweb + '/' + dirname
        ui.debug(debugtemplate % (defaulturl, defaultweb, defaultp, default, url))

        # real work
        node = bin(node)
        start = repo.changelog.rev(node)
        end = repo.changelog.count()
        for rev in xrange(start, end):
            tmp = repo.changelog.node(rev)
            ui.write(url, "/rev/%s - %s\n" % (short(tmp), repo.changelog.read(tmp)[4]))

## END OF FILE #################################################################
# vim: filetype=python expandtab
